
from traitsui.api import View


class UpdateInfoView(View):
    """ A View for UpdateInfo objects that displays them in a nice HTML widget.
    Note that this view does NOT include any UI widgets for interacting with
    the user or prompting for a download; for that, use the UpdateCheckerDialog.
    """
    pass

